Index: main/config.c
===================================================================
--- main/config.c	(revision 96620)
+++ main/config.c	(working copy)
@@ -1196,7 +1196,7 @@
 		if (cfmtime)
 			cfmtime->mtime = statbuf.st_mtime;
 
-		ast_verb(2, "Parsing '%s': ", fn);
+		ast_verb(2, "Parsing '%s': \n", fn);
 			fflush(stdout);
 		if (!(f = fopen(fn, "r"))) {
 			ast_debug(1, "No file to parse: %s\n", fn);
Index: res/res_musiconhold.c
===================================================================
--- res/res_musiconhold.c	(revision 96620)
+++ res/res_musiconhold.c	(working copy)
@@ -906,23 +906,6 @@
 	return class->total_files;
 }
 
-static int moh_diff(struct mohclass *old, struct mohclass *new)
-{
-	if (!old || !new)
-		return -1;
-
-	if (strcmp(old->dir, new->dir))
-		return -1;
-	else if (strcmp(old->mode, new->mode))
-		return -1;
-	else if (strcmp(old->args, new->args))
-		return -1;
-	else if (old->flags != new->flags)
-		return -1;
-
-	return 0;
-}
-
 static int moh_register(struct mohclass *moh, int reload)
 {
 #ifdef HAVE_ZAPTEL
@@ -931,12 +914,13 @@
 	struct mohclass *mohclass = NULL;
 
 	AST_RWLIST_WRLOCK(&mohclasses);
-	if ((mohclass = get_mohbyname(moh->name, 0)) && !moh_diff(mohclass, moh)) {
-		mohclass->delete = 0;
-		if (reload) {
-			ast_debug(1, "Music on Hold class '%s' left alone from initial load.\n", moh->name);
-		} else {
+	if ((mohclass = get_mohbyname(moh->name, 0)) ) {
+		if ( !mohclass->delete ) {
+			mohclass->delete = 0;
 			ast_log(LOG_WARNING, "Music on Hold class '%s' already exists\n", moh->name);
+			free(moh);	
+			AST_RWLIST_UNLOCK(&mohclasses);
+			return -1;
 		}
 		ast_free(moh);	
 		AST_RWLIST_UNLOCK(&mohclasses);
@@ -1548,7 +1532,7 @@
 
 static int reload(void)
 {
-	if (init_classes(1))
+	if (init_classes(0))
 		ast_install_music_functions(local_ast_moh_start, local_ast_moh_stop, local_ast_moh_cleanup);
 
 	return 0;
@@ -1556,7 +1540,31 @@
 
 static int unload_module(void)
 {
-	return -1;
+        int res = 0;
+        struct mohclass *class = NULL;
+
+	AST_RWLIST_RDLOCK(&mohclasses);
+	AST_RWLIST_TRAVERSE(&mohclasses, class, list) {
+		if (class->inuse > 0) {
+			res = -1;
+			break;
+		}
+	}
+	AST_RWLIST_UNLOCK(&mohclasses);
+	if (res < 0) {
+		ast_log(LOG_WARNING, "Unable to unload res_musiconhold due to active MOH channels\n");
+		return res;
+	}
+
+	ast_uninstall_music_functions();
+	ast_moh_destroy();
+	res = ast_unregister_application(play_moh);
+	res |= ast_unregister_application(wait_moh);
+	res |= ast_unregister_application(set_moh);
+	res |= ast_unregister_application(start_moh);
+	res |= ast_unregister_application(stop_moh);
+	res |= ast_cli_unregister_multiple(cli_moh, sizeof(cli_moh) / sizeof(struct ast_cli_entry));
+	return res;
 }
 
 AST_MODULE_INFO(ASTERISK_GPL_KEY, AST_MODFLAG_DEFAULT, "Music On Hold Resource",
